# frozen_string_literal: true
require 'test_helper'

require 'eaternet'
require 'eaternet/lives_1_0'

CASSETTE = 'nyc'
MORRIS_PARK_CSV = 'test/fixtures/nyc.csv'

class NycAdapterTest < Minitest::Test
  @@nyc = Eaternet::Nyc.new(csv_path: MORRIS_PARK_CSV)

  def setup
    @may_may_data = {
      'CAMIS' =>    '40358429',
      'DBA' =>      'MAY MAY KITCHEN',
      'BUILDING' => '1269      ',
      'STREET' =>   'SUTTER AVENUE                                     ',
      'BORO' =>     'BROOKLYN',
      'ZIPCODE' =>  '11208',
      'PHONE' =>    '7182354485'
    }
  end

  #
  # Businesses
  #

  def test_businesses_returns_an_enumerator_of_business
    assert enumerable_of? Eaternet::Lives_1_0::Business, @@nyc.businesses
  end

  def test_businesses_returns_lives_attributes
    b = @@nyc.businesses.first
    assert_equal '30075445', b.business_id
    assert_equal 'MORRIS PARK BAKE SHOP', b.name
    assert_equal '1007 MORRIS PARK AVE', b.address
    assert_equal 'BRONX', b.city
    assert_equal '10462', b.postal_code
    assert_equal 'NY', b.state
    assert_equal '7188924968', b.phone_number
  end

  def test_can_create_an_array_from_businesses
    assert_instance_of Array, @@nyc.businesses.to_a
  end

  def test_businesses_returns_unique_rows
    assert_equal 1, @@nyc.businesses.to_a.size
  end

  #
  # Inspections
  #

  def test_inspections_returns_an_enumerable_of_inspection
    assert enumerable_of? Eaternet::Lives_1_0::Inspection, @@nyc.inspections
  end

  # Use the wiki analysis document as a test.
  # @see https://github.com/eaternet/adapters/wiki/Agency:-New-York-City
  def test_inspections_finds_correct_dates
    expected_dates = %w(
      02/09/2015
      03/03/2014
      09/11/2013
      08/14/2013
      01/24/2013
      12/31/2012
    ).map { |d| Date.strptime(d, '%m/%d/%Y') }
    actual_dates = @@nyc.inspections.map(&:date).to_a
    assert_equal expected_dates, actual_dates
  end

  def test_inspections_returns_lives_attributes
    i = @@nyc.inspections.first(6).last
    assert_equal '30075445', i.business_id
    assert_equal Date.new(2012, 12, 31), i.date
    assert_equal 75, i.score
  end


  #
  # Violations
  #

  def test_violations_returns_an_enumerable_of_violation
    assert enumerable_of? Eaternet::Lives_1_0::Violation, @@nyc.violations
  end

  def test_violations_returns_lives_attributes
    v = @@nyc.violations.first
    assert_equal '30075445', v.business_id
    assert_equal Date.new(2015, 2, 9), v.date
    assert_equal '06C', v.code
    assert_match /Food not protected/, v.description
  end

  def test_finds_correct_violations
    expected_violations = %w(06C 10F 04L 04N 04C 04L 06A 06C 08A
                             10F 02G 10F 02G 04L 06C 08A 10F)
    actual_violations = @@nyc.violations.to_a.map(&:code)
    assert_equal expected_violations, actual_violations
  end

  #
  # metadata
  #

  def test_implements_feed_info
    assert_instance_of Eaternet::Lives_1_0::FeedInfo, @@nyc.feed_info
  end

  def test_has_legends
    assert enumerable_of? Eaternet::Lives_1_0::Legend, @@nyc.legends
  end
end
