require 'test_helper'

require 'eaternet/lives_1_0'

class LegendTest < Minitest::Test
  def setup_required_attrs(legend_record)
    legend_record.minimum_score = 98
    legend_record.maximum_score = 100
    legend_record.description =   'A+'
  end

  def setup
    @a_plus = Eaternet::Lives_1_0::Legend.new { |l| setup_required_attrs(l) }
  end


  def test_with_all_required_attributes
    assert @a_plus.valid?
  end

  def test_rejects_incorrect_minimum_type
    @a_plus.minimum_score = '98'
    refute @a_plus.valid?
  end

  def test_rejects_incorrect_maximum_type
    @a_plus.maximum_score = '100'
    refute @a_plus.valid?
  end

  def test_rejects_when_missing_maximum_score
    assert_raises(ArgumentError) {
      Eaternet::Lives_1_0::Legend.new do |legend_record|
        legend_record.minimum_score = 100
        legend_record.description =   'A+'
      end
    }
  end

  def test_rejects_when_missing_minimum_score
    assert_raises(ArgumentError) {
      Eaternet::Lives_1_0::Legend.new do |legend_record|
        legend_record.maximum_score = 100
        legend_record.description =   'A+'
      end
    }
  end

  def test_rejects_when_missing_description
    assert_raises(ArgumentError) {
      Eaternet::Lives_1_0::Legend.new do |legend_record|
        legend_record.minimum_score = 95
        legend_record.maximum_score = 100
      end
    }
  end

  def test_rejects_when_description_not_a_string
    @a_plus.description = %w(the highest score)
    refute @a_plus.valid?
  end

  #
  # LegendGroup
  #

  def test_group_accepts_comprehensive_records
    group = Eaternet::Lives_1_0::LegendGroup.new do |lg|
      lg.legends = [
        Eaternet::Lives_1_0::Legend.new do |l|
          l.minimum_score = 95
          l.maximum_score = 100
          l.description =   'A+'
        end,
        Eaternet::Lives_1_0::Legend.new do |l|
          l.minimum_score = 0
          l.maximum_score = 94
          l.description =   'F'
        end
      ]
    end
    refute_nil group
  end

  def test_group_accepts_records_overlapping_by_1
    group = Eaternet::Lives_1_0::LegendGroup.new do |lg|
      lg.legends = [
        Eaternet::Lives_1_0::Legend.new do |l|
          l.minimum_score = 95
          l.maximum_score = 100
          l.description =   'A+'
        end,
        Eaternet::Lives_1_0::Legend.new do |l|
          l.minimum_score = 70
          l.maximum_score = 95
          l.description =   'B'
        end,
        Eaternet::Lives_1_0::Legend.new do |l|
          l.minimum_score = 0
          l.maximum_score = 69
          l.description =   'F'
        end
      ]
    end
    refute_nil group
  end

  def test_legend_group_rejects_non_comprehensive_records
    assert_raises(ArgumentError) {
      Eaternet::Lives_1_0::LegendGroup.new do |lg|
        lg.legends = [@a_plus]
      end
    }
  end

  def test_legend_group_rejects_overlapping_records
    assert_raises(ArgumentError) {
      Eaternet::Lives_1_0::LegendGroup.new do |lg|
        lg.legends = [
          Eaternet::Lives_1_0::Legend.new do |l|
            l.minimum_score = 90
            l.maximum_score = 100
            l.description =   'A'
          end,
          Eaternet::Lives_1_0::Legend.new do |l|
            l.minimum_score = 0
            l.maximum_score = 91
            l.description =   'B'
          end
        ]
      end
    }
  end

  def test_legend_group_rejects_non_legend_objects
    assert_raises(ArgumentError) {
      Eaternet::Lives_1_0::LegendGroup.new do |lg|
        lg.legends = [
          Eaternet::Lives_1_0::Legend.new do |l|
            l.minimum_score = 90
            l.maximum_score = 100
            l.description =   'A'
          end,
          '0-91, B'
        ]
      end
    }
  end
end
